@@ -26,8 +26,7 @@
 #include <telepathy-glib/channel.h>
 #include <telepathy-glib/dbus.h>
 #include <telepathy-glib/util.h>
-
-#include "empathy-account.h"
+#include <telepathy-glib/interfaces.h>
 
 #include "empathy-tp-roomlist.h"
 #include "empathy-chatroom.h"
@@ -40,7 +39,7 @@
 typedef struct {
 	TpConnection *connection;
 	TpChannel    *channel;
-	EmpathyAccount    *account;
+	TpAccount    *account;
 	gboolean      is_listing;
 	gboolean      start_requested;
 } EmpathyTpRoomlistPriv;
@@ -262,67 +261,49 @@ stop_listing_cb (TpChannel *proxy,
 }
 
 static void
-channel_ready_cb (TpChannel *channel,
-		  const GError *error,
-		  gpointer user_data)
+tp_roomlist_create_channel_cb (GObject *source,
+				GAsyncResult *result,
+				gpointer      user_data)
 {
-	EmpathyTpRoomlist *list = EMPATHY_TP_ROOMLIST (user_data);
-	EmpathyTpRoomlistPriv *priv = GET_PRIV (list);
+	EmpathyTpRoomlist *self = user_data;
+	EmpathyTpRoomlistPriv *priv = GET_PRIV (self);
+	GError *error = NULL;
 
-	if (error != NULL) {
-		DEBUG ("Channel invalidated: %s", error->message);
-		g_signal_emit (list, signals[DESTROY], 0);
+	priv->channel = tp_account_channel_request_create_and_handle_channel_finish (
+		TP_ACCOUNT_CHANNEL_REQUEST (source), result, NULL, &error);
+
+	if (priv->channel == NULL) {
+		DEBUG ("Error creating channel: %s", error->message);
+		g_error_free (error);
 		return;
 	}
 
 	g_signal_connect (priv->channel, "invalidated",
-			  G_CALLBACK (tp_roomlist_invalidated_cb),
-			  list);
+			  G_CALLBACK (tp_roomlist_invalidated_cb), self);
 
 	tp_cli_channel_type_room_list_connect_to_listing_rooms (priv->channel,
 								tp_roomlist_listing_cb,
 								NULL, NULL,
-								G_OBJECT (list),
+								G_OBJECT (self),
 								NULL);
 	tp_cli_channel_type_room_list_connect_to_got_rooms (priv->channel,
 							    tp_roomlist_got_rooms_cb,
 							    NULL, NULL,
-							    G_OBJECT (list),
+							    G_OBJECT (self),
 							    NULL);
 
 	tp_cli_channel_type_room_list_call_get_listing_rooms (priv->channel, -1,
 							      tp_roomlist_get_listing_rooms_cb,
 							      NULL, NULL,
-							      G_OBJECT (list));
+							      G_OBJECT (self));
 
 	if (priv->start_requested == TRUE) {
 		tp_cli_channel_type_room_list_call_list_rooms (priv->channel, -1,
-			call_list_rooms_cb, list, NULL, NULL);
+			call_list_rooms_cb, self, NULL, NULL);
 		priv->start_requested = FALSE;
 	}
 }
 
-static void
-tp_roomlist_request_channel_cb (TpConnection *connection,
-				const gchar  *object_path,
-				const GError *error,
-				gpointer      user_data,
-				GObject      *list)
-{
-	EmpathyTpRoomlistPriv *priv = GET_PRIV (list);
-
-	if (error) {
-		DEBUG ("Error requesting channel: %s", error->message);
-		return;
-	}
-
-	priv->channel = tp_channel_new (priv->connection, object_path,
-					TP_IFACE_CHANNEL_TYPE_ROOM_LIST,
-					TP_HANDLE_TYPE_NONE,
-					0, NULL);
-	tp_channel_call_when_ready (priv->channel, channel_ready_cb, list);
-}
-
 static void
 tp_roomlist_finalize (GObject *object)
 {
@@ -352,18 +333,26 @@ static void
 tp_roomlist_constructed (GObject *list)
 {
 	EmpathyTpRoomlistPriv *priv = GET_PRIV (list);
+	GHashTable *request;
+	TpAccountChannelRequest *req;
 
-	priv->connection = empathy_account_get_connection (priv->account);
+	request = tp_asv_new (
+		TP_PROP_CHANNEL_CHANNEL_TYPE, G_TYPE_STRING,
+		  TP_IFACE_CHANNEL_TYPE_ROOM_LIST,
+		TP_PROP_CHANNEL_TARGET_HANDLE_TYPE, G_TYPE_UINT, TP_HANDLE_TYPE_NONE,
+		NULL);
+
+	priv->connection = tp_account_get_connection (priv->account);
 	g_object_ref (priv->connection);
 
-	tp_cli_connection_call_request_channel (priv->connection, -1,
-						TP_IFACE_CHANNEL_TYPE_ROOM_LIST,
-						TP_HANDLE_TYPE_NONE,
-						0,
-						TRUE,
-						tp_roomlist_request_channel_cb,
-						NULL, NULL,
-						list);
+	req = tp_account_channel_request_new (priv->account, request,
+		TP_USER_ACTION_TIME_CURRENT_TIME);
+
+	tp_account_channel_request_create_and_handle_channel_async (req, NULL,
+		tp_roomlist_create_channel_cb, list);
+
+	g_hash_table_unref (request);
+	g_object_unref (req);
 }
 
 static void
@@ -420,7 +409,7 @@ empathy_tp_roomlist_class_init (EmpathyTpRoomlistClass *klass)
 					 g_param_spec_object ("account",
 							      "The Account",
 							      "The account on which it lists rooms",
-							      EMPATHY_TYPE_ACCOUNT,
+							      TP_TYPE_ACCOUNT,
 							      G_PARAM_READWRITE |
 							      G_PARAM_CONSTRUCT_ONLY));
 	g_object_class_install_property (object_class,
@@ -476,7 +465,7 @@ empathy_tp_roomlist_init (EmpathyTpRoomlist *list)
 }
 
 EmpathyTpRoomlist *
-empathy_tp_roomlist_new (EmpathyAccount *account)
+empathy_tp_roomlist_new (TpAccount *account)
 {
 	EmpathyTpRoomlist *list;
 
@@ -517,6 +506,10 @@ empathy_tp_roomlist_stop (EmpathyTpRoomlist *list)
 	EmpathyTpRoomlistPriv *priv = GET_PRIV (list);
 
 	g_return_if_fail (EMPATHY_IS_TP_ROOMLIST (list));
+
+	if (priv->channel == NULL)
+		return;
+
 	g_return_if_fail (TP_IS_CHANNEL (priv->channel));
 
 	tp_cli_channel_type_room_list_call_stop_listing (priv->channel, -1,
